import { Application } from 'express'

import { AssetsController } from '#http/controllers/assets'
import { AuthController } from '#http/controllers/auth'
import { AvatarController } from '#http/controllers/avatar'

class HttpManager {
  private readonly authController: AuthController
  private readonly avatarController: AvatarController
  private readonly assetsController: AssetsController

  constructor() {
    this.authController = new AuthController()
    this.avatarController = new AvatarController()
    this.assetsController = new AssetsController()
  }

  public async boot(app: Application) {
    // Add routes
    await this.addRoutes(app)
  }
  
  private async addRoutes(app: Application) {
    // Auth routes
    app.post('/login', (req, res) => this.authController.login(req, res))
    app.post('/register', (req, res) => this.authController.register(req, res))
    app.post('/reset-password-request', (req, res) => this.authController.requestPasswordReset(req, res))
    app.post('/reset-password', (req, res) => this.authController.resetPassword(req, res))

    // Avatar routes
    app.get('/avatar/:characterName', (req, res) => this.avatarController.getByName(req, res))
    app.get('/avatar/s/:characterTypeId/:characterHairId?', (req, res) => this.avatarController.getByParams(req, res))

    // Assets routes
    app.get('/assets/list_tiles', (req, res) => this.assetsController.listTiles(req, res))
    app.get('/assets/list_tiles/:zoneId', (req, res) => this.assetsController.listTilesByZone(req, res))
    app.get('/assets/list_sprite_actions/:spriteId', (req, res) => this.assetsController.listSpriteActions(req, res))
    app.get('/assets/:type/:spriteId?/:file', (req, res) => this.assetsController.downloadAsset(req, res))
  }
}

export default new HttpManager()